@@ -465,6 +465,8 @@ class _CorruptedHDU(_BaseHDU):
         Returns the size (in bytes) of the HDU's data part.
         """
 
+        # Note: On compressed files this might report a negative size; but the
+        # file is corrupt anyways so I'm not too worried about it.
         return self._file.size - self._datLoc
 
     def _summary(self):
@@ -860,10 +862,12 @@ class _ValidHDU(_BaseHDU, _Verify):
             # if the supposed location is specified
             if pos is not None:
                 if not pos(_index):
-                    err_text = "'%s' card at the wrong place (card %d)." \
-                               % (keyword, _index)
-                    fix_text = "Fixed by moving it to the right place " \
-                               "(card %d)." % insert_pos
+                    err_text = ("'%s' card at the wrong place (card %d) "
+                                "(note: PyFITS uses zero-based indexing)." %
+                                (keyword, _index))
+                    fix_text = ("Fixed by moving it to the right place "
+                                "(card %d) (note: PyFITS uses zero-based "
+                                "indexing)." % insert_pos)
 
                     def fix(self=self, index=_index, insert_pos=insert_pos):
                         cards = self._header.ascard
