@@ -56,7 +56,7 @@ class TestImageFunctions(PyfitsTestCase):
         """Test string value"""
 
         c = pyfits.Card('abc', '<8 ch')
-        assert_equal(str(c), 
+        assert_equal(str(c),
                      "ABC     = '<8 ch   '                                                            ")
         c = pyfits.Card('nullstr', '')
         assert_equal(str(c),
@@ -84,7 +84,7 @@ class TestImageFunctions(PyfitsTestCase):
 
         c = pyfits.Card('floatnum', -467374636747637647347374734737437.)
 
-        if (str(c) != "FLOATNUM= -4.6737463674763E+32                                                  " and 
+        if (str(c) != "FLOATNUM= -4.6737463674763E+32                                                  " and
             str(c) != "FLOATNUM= -4.6737463674763E+032                                                 "):
             assert_equal(str(c),
                          "FLOATNUM= -4.6737463674763E+32                                                  ")
@@ -216,9 +216,9 @@ class TestImageFunctions(PyfitsTestCase):
         c = pyfits.Card.fromstring('abc= a6')
         with CaptureStdout() as f:
             c.verify()
-            assert_equal(f.getvalue(),
-                'Output verification result:\n'
-                'Card image is not FITS standard (equal sign not at column 8).\n')
+            assert_true(
+                'Card image is not FITS standard (equal sign not at column 8).'
+                in f.getvalue())
         assert_equal(str(c),
                      "abc= a6                                                                         ")
 
@@ -226,9 +226,7 @@ class TestImageFunctions(PyfitsTestCase):
         c = pyfits.Card.fromstring('abc= a6')
         with CaptureStdout() as f:
             c.verify('fix')
-            assert_equal(f.getvalue(),
-                         'Output verification result:\n'
-                         '  Fixed card to be FITS standard.: ABC\n')
+            assert_true('Fixed card to be FITS standard.: ABC' in f.getvalue())
         assert_equal(str(c),
                      "ABC     = 'a6      '                                                            ")
 
@@ -259,7 +257,7 @@ class TestImageFunctions(PyfitsTestCase):
             "CONTINUE  '&' / long comment long comment long comment long comment long        \n"
             "CONTINUE  '&' / comment long comment long comment long comment long comment     \n"
             "CONTINUE  '&' / long comment                                                    ")
- 
+
 
     def test_word_in_long_string_too_long(self):
         # if a word in a long string is too long, it will be cut in the middle
@@ -474,16 +472,14 @@ class TestImageFunctions(PyfitsTestCase):
             x = pyfits.ImageHDU()
             hdu = pyfits.HDUList(x) # HDUList can take a list or one single HDU
             hdu.verify()
-            assert_equal(f.getvalue(),
-                "Output verification result:\n"
-                "HDUList's 0th element is not a primary HDU.\n")
+            assert_true(
+                "HDUList's 0th element is not a primary HDU." in f.getvalue())
 
         with CaptureStdout() as f:
             hdu.writeto(self.temp('test_new2.fits'), 'fix')
-            assert_equal(f.getvalue(),
-                "Output verification result:\n"
+            assert_true(
                 "HDUList's 0th element is not a primary HDU.  "
-                "Fixed by inserting one as 0th HDU.\n")
+                "Fixed by inserting one as 0th HDU." in f.getvalue())
 
     def test_section(self):
         # section testing
@@ -716,6 +712,7 @@ class TestImageFunctions(PyfitsTestCase):
         ofd.close()
         fd = pyfits.open(self.temp('test_new.fits'))
         assert_equal(fd[1].data.all(), data.all())
+        fd.close()
 
         data = np.zeros((100, 100)) + 1
         chdu = pyfits.CompImageHDU(data)
@@ -774,3 +771,23 @@ class TestImageFunctions(PyfitsTestCase):
         hdul = pyfits.open(self.temp('test_new.fits'))
         arr += 1
         assert_true((hdul[1].data == arr).all())
+
+    def test_rewriting_large_scaled_image(self):
+        """Regression test for #84"""
+
+        hdul = pyfits.open(self.data('fixed-1890.fits'))
+        orig_data = hdul[0].data
+        hdul.writeto(self.temp('test_new.fits'), clobber=True)
+        hdul.close()
+        hdul = pyfits.open(self.temp('test_new.fits'))
+        assert_true((hdul[0].data == orig_data).all())
+        hdul.close()
+
+        # Just as before, but this time don't touch hdul[0].data before writing
+        # back out--this is the case that failed in #84
+        hdul = pyfits.open(self.data('fixed-1890.fits'))
+        hdul.writeto(self.temp('test_new.fits'), clobber=True)
+        hdul.close()
+        hdul = pyfits.open(self.temp('test_new.fits'))
+        assert_true((hdul[0].data == orig_data).all())
+        hdul.close()
